#!/usr/bin/python
# -*- coding: utf-8 -*-

from scannedonlyd import ScannedOnly, SCANNEDONLY, Options

class Scanner(object):
    def __init__(self, sock):
        self.rconn = sock.makefile('r', 1)
        self.conn = sock

    def read_fd(self):
        return self.rconn.fileno()

    def write_fd(self):
        return self.conn.fileno()

    def produce(self, path):
        #
        # See http://www.f-prot.com/support/helpfiles/unix/appendix_a.html for the
        # f-prot protocol specification.
        #
        self.conn.sendall('SCAN FILE %s\n' % path)
        return path

    def consume(self, path):
        #
        # The result string is of the form:
        # n ' <' descriptive text '> ' name of the scanned object
        # Where n is the result code.
        #
        # See http://www.f-prot.com/support/helpfiles/unix/appendix_c.html for
        # all the possible f-prot return codes.
        #

        VIR_MASK = 0x03
        ERR_MASK = 0x3c

        line = self.rconn.readline()[:-1]     # chomp '\n'
        result_code, result = line.split(' <', 1)
        result_code = int(result_code)

        descr, _ = result.split('> ', 1)
        if result_code & ERR_MASK:
            return SCANNEDONLY.FAILED, path, descr
        if result_code & VIR_MASK:
            return SCANNEDONLY.INFECTED, path, descr
        return SCANNEDONLY.CLEAN, path, descr

class Fprotd(object):
    def __init__(self, opts):
        self.opts = opts

    def setup(self):
        pass

    def scanners(self):
        from socket import socket, error
        sock = socket()
        for port in self.opts.fprot_ports:
            try:
                for i in range(self.opts.maxthreads):
                    sock.connect((self.opts.fprot_ip, port))
                    yield Scanner(sock)
                    sock = socket()
            except error:
                continue

if __name__ == '__main__':
    from optparse import Option
    ip = Option('-I', '--fprot_ip', help="IP of F-Prot daemon "
                "(default: %default)", default="127.0.0.1")
    ports = Option('-P', '--fprot_ports', action="append", type="int",
                   help="port number(s), F-Prot daemon listens on "
                   "(default: %default)",
                   default=[10200,10201,10202,10203,10204])
    opts, args = Options(ip, ports)
    ScannedOnly(opts)(Fprotd(opts))

